#    most of this task is now taken from the standard application

set VERSION "CAEN"

#    put some default environment variables before loading common 

if {[info exists env(RDOUThdwre)] == 0} then {set env(RDOUThdwre) VME} 

loadsource  sys.common

set frametitle "MIDAS Data Acquisition Experiment Control (CAEN)"

set VMEPresent 1
set VMEEnable 1

set TSEnable 0

set histEnable 1
set HistMode 1

set txEnable 1

set TestBuffer ""

#    some additional registers for the CAEN
lappend registerlist [list $RDOUThdwre.Verbose1   generic  "-nVerbose1 -tint"]
lappend registerlist [list $RDOUThdwre.Verbose2   generic  "-nVerbose2 -tint"]
lappend registerlist [list $RDOUThdwre.Verbose3   generic  "-nVerbose3 -tint"]

lappend actionlist [list "Show histogram name translations" "Show_Xlate"]
lappend actionlist [list "Edit histogram translation file"  "edit_xlate"]

proc SpectrumName {n} {Give_Xlate $n}

proc create_spectra {} {
   global SpectrumServer HistMode EGreport

   if {$HistMode == 0} {return}

   set-footer "Creating spectra"
   set-busy

   Make_Xlate $SpectrumServer

   set EGreport_SAVE $EGreport
   set EGreport -1

   set maxmodule [inform vmecontrol inquire v785count]
   set range [expr ($maxmodule+1)*64]

   set z [catch {eg "(spectrum :path '/$SpectrumServer' :range $range :name 'Stat' !new)" } m]
   map_spectrum "Stat 0 0"

   set z [catch {eg "(spectrum :path '/$SpectrumServer' :range $range :name 'Rate' !new)" } m]

   set k 2

#   for CAEN V785

   set maxadc 32

   for {set i 1} {$i <= $maxmodule} {incr i} {
    for {set j 0} {$j < $maxadc} {incr j} {

      set spectrumname [SpectrumName v785#$i.adc$j]

      set z [catch {eg "(spectrum :path '/$SpectrumServer' :range 4096 :name '$spectrumname' !new)" } m]

#    Allow "Spectrum already exists" so that further spectra can be created
      if {$z!=0 && $m==0x30004} then {set z 0}
      if {$z!=0} then {
          display-eg-error-report self $m "Attempting to create spectrum $spectrumname"
          clear-footer
          error $m
      } else {
          incr k
          map_spectrum  "$spectrumname $i $j"
      }
    }
   }

   set EGreport $EGreport_SAVE

   clear-busy
   set-footer "$k spectra created"

}

proc edit_xlate {}  {
   global XlatePres Xlate
   global env SpectrumServer

   set maxmodule [inform vmecontrol inquire v785count]
   set maxadc 32

   if {$XlatePres == 0} then {Make_Xlate $SpectrumServer}

   set contents ""

   for {set m 1} {$m <= $maxmodule} {incr m} {
      for {set c 0} {$c < $maxadc} {incr c} {

         set s v785#$m.adc$c
         if {$s != [SpectrumName $s]} then {
             append contents [format "%24s %24s\n" $s $Xlate($s)]
         } else {
             append contents [format "%24s %24s\n" $s ""]
         }
      }
   }

    set contents [split [Edit_Xlate $contents] \n]

    if {[llength $contents] == 0} then {return}

#    data has been modified  - save to file

    set file [file join $env(EXPERIMENTBASE) Histconfigs [set SpectrumServer]_names]
    set z [catch {open $file w} f]
    if {$z != 0} then {midas-warning "Unable to write file $file\n$f"; return}

    foreach item $contents {
       if {[llength $item] == 2} then {
           puts $f "$item"
       }
    }
    close $f
    set-footer "$file updated"

#  update the arrays

    Make_Xlate $SpectrumServer
}

proc define_VME_addresses {} {
    global VMEserver

#    load hardware configuration into readout software

   ClearConfig

#    first all the ADCs

   set maxmodule [inform vmecontrol inquire v785count]
   for {set i 1} {$i <= $maxmodule} {incr i} {

      set z [catch {eg "(register :name '/$VMEserver/v785#$i.GEO_Address'  ?object)" } m]
      if {$z!=0} then {
          display-eg-error-report self $m "Attempting to locate register v785#$i.GEO_Address"
          clear-footer
          error $m
      }
      DefineConfig "785 $i [string range [first $m] 2 end]";
   }

#    then all the TDCs

   set maxmodule [inform vmecontrol inquire v767count]
   for {set i 1} {$i <= $maxmodule} {incr i} {

      set z [catch {eg "(register :name '/$VMEserver/v767#$i.GEO_Address'  ?object)" } m]
      if {$z!=0} then {
          display-eg-error-report self $m "Attempting to locate register v767#$i.GEO_Address"
          clear-footer
          error $m
      }
      DefineConfig "767 [expr 8 + $i] [string range [first $m] 2 end]";
   }

}

proc DefineConfig {s} {

   global RDOUTEnable; if {$RDOUTEnable == 0} then {return}

#    format of module configuration record is  type  module_number  VME_address

   write-register EXEC.Parameter $s
   write-register EXEC.Action 13
   write-register EXEC.Sig1 0
   wait_for_completion "Define Configuration"
}

proc click-vme {action} {

   starttask vmecontrol

    switch $action {
      0    {set p DAQ-halt}
      1    {set p DAQ-go}
      2    {set p DAQ-reset}
      3    {set p DAQ-setup}
    }
 
   set-footer "sending $p to VME control task"
   set-busy

   set z [catch {send vmecontrol $p} m]

   clear-busy
   if {$m == ""} then {clear-footer} else {set-footer $m}

   if {$action == 3} then {define_VME_addresses}

   redisplay-vme
}

